@@ -23,13 +23,13 @@ set -x
 
 requires $QEMU_NBD --pid-file=test.pid --version
 requires qemu-img --version
-requires cmp --version
-requires dd --version
-requires dd oflag=seek_bytes </dev/null
-requires stat --version
+requires $CMP --version
+requires $DD --version
+requires $DD oflag=seek_bytes </dev/null
+requires $STAT --version
 requires test -r /dev/urandom
 requires test -r /dev/zero
-requires truncate --version
+requires $TRUNCATE --version
 
 file=copy-file-to-qcow2.file
 file2=copy-file-to-qcow2.file2
@@ -42,12 +42,12 @@ cleanup_fn rm -f $file $file2 $qcow2 $pid $sock
 # Create a random partially sparse file.
 touch $file
 for i in `seq 1 100`; do
-    dd if=/dev/urandom of=$file ibs=512 count=1 \
-       oflag=seek_bytes seek=$((RANDOM * 9973)) conv=notrunc
-    dd if=/dev/zero of=$file ibs=512 count=1 \
-       oflag=seek_bytes seek=$((RANDOM * 9973)) conv=notrunc
+    $DD if=/dev/urandom of=$file ibs=512 count=1 \
+        oflag=seek_bytes seek=$((RANDOM * 9973)) conv=notrunc
+    $DD if=/dev/zero of=$file ibs=512 count=1 \
+        oflag=seek_bytes seek=$((RANDOM * 9973)) conv=notrunc
 done
-size="$( stat -c %s $file )"
+size="$( $STAT -c %s $file )"
 
 # Create the empty target qcow2 file.
 qemu-img create -f qcow2 $qcow2 $size
@@ -66,6 +66,6 @@ ls -l $file $file2
 
 # Because qemu/qcow2 only supports whole sectors, we have to truncate
 # the copied out file to the expected size before comparing.
-truncate -s $size $file2
+$TRUNCATE -s $size $file2
 
-cmp $file $file2
+$CMP $file $file2
