@@ -205,32 +205,50 @@ page_cache_evict (struct rw_file *rwf, uint64_t orig_offset, size_t orig_len)
 
 struct write_window {
   uint64_t offset;
-  size_t len;
+  size_t len;           /* window slot only valid if len > 0 */
 };
 
-static inline void
-evict_writes (struct rw_file *rwf, uint64_t offset, size_t len)
+static void
+evict_writes (int fd, uint64_t offset, size_t len)
 {
   static __thread struct write_window window[NR_WINDOWS];
+  struct write_window oldest = { 0 };
 
-  /* Evict the oldest window from the page cache. */
-  if (window[0].len > 0) {
-    sync_file_range (rwf->fd, window[0].offset, window[0].len,
-                     SYNC_FILE_RANGE_WAIT_BEFORE|SYNC_FILE_RANGE_WRITE|
-                     SYNC_FILE_RANGE_WAIT_AFTER);
-    posix_fadvise (rwf->fd, window[0].offset, window[0].len,
-                   POSIX_FADV_DONTNEED);
+  /* Save oldest window[0] for eviction below, and move all windows
+   * down one.  Set the newest slot to empty.
+   */
+  oldest = window[0];
+  memmove (&window[0], &window[1], sizeof window[0] * (NR_WINDOWS-1));
+  window[NR_WINDOWS-1].len = 0;
+
+  /* Tell Linux to start writing the current range out to disk
+   * (asynchronously).
+   */
+  if (sync_file_range (fd, offset, len, SYNC_FILE_RANGE_WRITE) == -1) {
+    fprintf (stderr, "%s: sync_file_range: cache=none: "
+             "starting eviction: %m", prog);
+    exit (EXIT_FAILURE);
   }
 
-  /* Move the Nth window to N-1. */
-  memmove (&window[0], &window[1], sizeof window[0] * (NR_WINDOWS-1));
-
-  /* Set up the current window and tell Linux to start writing it out
-   * to disk (asynchronously).
-   */
-  sync_file_range (rwf->fd, offset, len, SYNC_FILE_RANGE_WRITE);
+  /* Add the range to the newest end of the list of windows. */
   window[NR_WINDOWS-1].offset = offset;
   window[NR_WINDOWS-1].len = len;
+
+  /* Evict the oldest window from the page cache (synchronously). */
+  if (oldest.len > 0) {
+    if (sync_file_range (fd, oldest.offset, oldest.len,
+                         SYNC_FILE_RANGE_WAIT_BEFORE |
+                         SYNC_FILE_RANGE_WRITE |
+                         SYNC_FILE_RANGE_WAIT_AFTER) == -1) {
+      fprintf (stderr, "%s: sync_file_range: cache=none: "
+               "evicting oldest window: %m", prog);
+      exit (EXIT_FAILURE);
+    }
+    if (posix_fadvise (fd, oldest.offset, oldest.len,
+                       POSIX_FADV_DONTNEED) == -1)
+      fprintf (stderr, "warning: posix_fadvise: "
+               "POSIX_FADV_DONTNEED: %m");
+  }
 }
 #endif /* EVICT_WRITES */
 
@@ -361,15 +379,16 @@ file_close (struct rw *rw)
   free (rw);
 }
 
+static void file_allocate_space (struct rw *rw, uint64_t size);
+
 static void
-file_truncate (struct rw *rw, int64_t size)
+file_truncate (struct rw *rw, int64_t size, bool allocate)
 {
   struct rw_file *rwf = (struct rw_file *)rw;
 
   /* If the destination is an ordinary file then the original file
    * size doesn't matter.  Truncate it to the source size.  But
-   * truncate it to zero first so the file is completely empty and
-   * sparse.
+   * truncate it to zero first so the entire file reads as zeroes.
    */
   if (rwf->is_block)
     return;
@@ -379,6 +398,10 @@ file_truncate (struct rw *rw, int64_t size)
     fprintf (stderr, "%s: truncate: %m\n", rw->name);
     exit (EXIT_FAILURE);
   }
+
+  if (allocate)
+    file_allocate_space (rw, size);
+
   rwf->rw.size = size;
 
   /* We can assume the target is zero. */
@@ -486,7 +509,7 @@ file_synch_write (struct rw *rw,
   }
 
 #if EVICT_WRITES
-  evict_writes (rwf, orig_offset, orig_len);
+  evict_writes (rwf->fd, orig_offset, orig_len);
 #endif
 }
 
@@ -555,6 +578,64 @@ file_zeroout (int fd, uint64_t offset, uint64_t count)
     return false;
 }
 
+static bool
+file_fallocate (int fd, uint64_t offset, uint64_t count)
+{
+#ifdef __linux__
+  int r;
+
+  r = fallocate (fd, 0, offset, count);
+  if (r == -1) {
+    if (is_not_supported (errno))
+      return false;
+
+    perror ("fallocate");
+    exit (EXIT_FAILURE);
+  }
+  return true;
+#endif
+  return false;
+}
+
+static void
+file_allocate_space (struct rw *rw, uint64_t size)
+{
+  struct rw_file *rwf = (struct rw_file *)rw;
+  void *buf;
+  uint64_t offset = 0;
+  uint64_t len = size;
+
+  /* Try efficient allocation if supported. */
+
+  if (rwf->can_fallocate) {
+    if (file_fallocate (rwf->fd, 0, size))
+      return;
+
+    rwf->can_fallocate = false;
+  }
+
+  /* Fallback to manually writing zeros. This is much faster than
+   * posix_fallocate() since we don't try to keep existing data.
+   */
+
+  buf = calloc (1, request_size);
+  if (buf == NULL) {
+    perror ("calloc");
+    exit (EXIT_FAILURE);
+  }
+
+  while (len > request_size) {
+    file_synch_write (rw, buf, request_size, offset);
+    offset += request_size;
+    len -= request_size;
+  }
+
+  if (len)
+    file_synch_write (rw, buf, len, offset);
+
+  free (buf);
+}
+
 static bool
 file_synch_zero (struct rw *rw, uint64_t offset, uint64_t count, bool allocate)
 {
@@ -593,10 +674,8 @@ file_synch_zero (struct rw *rw, uint64_t offset, uint64_t count, bool allocate)
 
   if (rwf->can_punch_hole && rwf->can_fallocate) {
     if (file_punch_hole (rwf->fd, offset, count)) {
-#ifdef __linux__
-      if (fallocate (rwf->fd, 0, offset, count))
+      if (file_fallocate (rwf->fd, offset, count))
           return true;
-#endif
 
       rwf->can_fallocate = false;
     } else {
