@@ -43,9 +43,11 @@ bool list_all = false;          /* --list option */
 bool probe_content = false;     /* --content / --no-content option */
 bool json_output = false;       /* --json option */
 const char *can = NULL;         /* --is/--can option */
+bool cannot = false;            /* --can option is negated */
 const char *map = NULL;         /* --map option */
 bool size_only = false;         /* --size option */
 bool totals = false;            /* --totals option */
+bool uri_only = false;          /* --uri option */
 
 /* See do_connect () */
 static enum { MODE_URI = 1, MODE_SQUARE_BRACKET } mode;
@@ -60,8 +62,11 @@ usage (FILE *fp, int exitcode)
 "\n"
 "    nbdinfo [--json] NBD-URI | [ CMD ARGS ... ]\n"
 "    nbdinfo --size [--json] NBD-URI | [ CMD ARGS ... ]\n"
+"    nbdinfo --uri [--json] NBD-URI\n"
 "    nbdinfo --is read-only|rotational NBD-URI | [ CMD ARGS ... ]\n"
+"    nbdinfo --isnt read-only|rotational NBD-URI | [ CMD ARGS ... ]\n"
 "    nbdinfo --can cache|connect|... NBD-URI | [ CMD ARGS ... ]\n"
+"    nbdinfo --cannot cache|connect|... NBD-URI | [ CMD ARGS ... ]\n"
 "    nbdinfo --map [--totals] [--json] NBD-URI | [ CMD ARGS ... ]\n"
 "    nbdinfo -L|--list [--json] NBD-URI | [ CMD ARGS ... ]\n"
 "\n"
@@ -75,6 +80,7 @@ usage (FILE *fp, int exitcode)
 "    nbdinfo nbd://localhost\n"
 "    nbdinfo \"nbd+unix:///?socket=/tmp/unixsock\"\n"
 "    nbdinfo --size nbd://example.com\n"
+"    nbdinfo --uri nbd://example.com\n"
 "    nbdinfo --can connect nbd://example.com\n"
 "    nbdinfo --is read-only nbd://example.com\n"
 "    nbdinfo --map nbd://example.com\n"
@@ -88,6 +94,17 @@ usage (FILE *fp, int exitcode)
   exit (exitcode);
 }
 
+void
+clean_shutdown (void)
+{
+  /* If we are connected but detect an error, try to give the server
+   * notice that we are done talking.  Ignore failures, as this is
+   * only a courtesy measure.
+   */
+  if (nbd)
+    nbd_shutdown (nbd, 0);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -101,14 +118,19 @@ main (int argc, char *argv[])
     NO_CONTENT_OPTION,
     JSON_OPTION,
     CAN_OPTION,
+    CANNOT_OPTION,
     MAP_OPTION,
     SIZE_OPTION,
     TOTALS_OPTION,
+    URI_OPTION,
   };
   const char *short_options = "LV";
   const struct option long_options[] = {
     { "help",               no_argument,       NULL, HELP_OPTION },
     { "can",                required_argument, NULL, CAN_OPTION },
+    { "cannot",             required_argument, NULL, CANNOT_OPTION },
+    { "can-not",            required_argument, NULL, CANNOT_OPTION },
+    { "cant",               required_argument, NULL, CANNOT_OPTION },
     { "color",              no_argument,       NULL, COLOUR_OPTION },
     { "colors",             no_argument,       NULL, COLOUR_OPTION },
     { "colour",             no_argument,       NULL, COLOUR_OPTION },
@@ -120,8 +142,17 @@ main (int argc, char *argv[])
     { "content",            no_argument,       NULL, CONTENT_OPTION },
     { "no-content",         no_argument,       NULL, NO_CONTENT_OPTION },
     { "has",                required_argument, NULL, CAN_OPTION },
+    { "hasnot",             required_argument, NULL, CANNOT_OPTION },
+    { "has-not",            required_argument, NULL, CANNOT_OPTION },
+    { "hasnt",              required_argument, NULL, CANNOT_OPTION },
     { "have",               required_argument, NULL, CAN_OPTION },
+    { "havent",             required_argument, NULL, CANNOT_OPTION },
+    { "havenot",            required_argument, NULL, CANNOT_OPTION },
+    { "have-not",           required_argument, NULL, CANNOT_OPTION },
     { "is",                 required_argument, NULL, CAN_OPTION },
+    { "isnot",              required_argument, NULL, CANNOT_OPTION },
+    { "is-not",             required_argument, NULL, CANNOT_OPTION },
+    { "isnt",               required_argument, NULL, CANNOT_OPTION },
     { "json",               no_argument,       NULL, JSON_OPTION },
     { "list",               no_argument,       NULL, 'L' },
     { "long-options",       no_argument,       NULL, LONG_OPTIONS },
@@ -130,6 +161,7 @@ main (int argc, char *argv[])
     { "size",               no_argument,       NULL, SIZE_OPTION },
     { "total",              no_argument,       NULL, TOTALS_OPTION },
     { "totals",             no_argument,       NULL, TOTALS_OPTION },
+    { "uri",                no_argument,       NULL, URI_OPTION },
     { "version",            no_argument,       NULL, 'V' },
     { NULL }
   };
@@ -190,6 +222,11 @@ main (int argc, char *argv[])
       can = optarg;
       break;
 
+    case CANNOT_OPTION:
+      can = optarg;
+      cannot = true;
+      break;
+
     case MAP_OPTION:
       map = optarg ? optarg : "base:allocation";
       break;
@@ -202,6 +239,10 @@ main (int argc, char *argv[])
       totals = true;
       break;
 
+    case URI_OPTION:
+      uri_only = true;
+      break;
+
     case 'L':
       list_all = true;
       break;
@@ -232,9 +273,10 @@ main (int argc, char *argv[])
   }
 
   /* You cannot combine certain options. */
-  if (!!list_all + !!can + !!map + !!size_only > 1) {
+  if (!!list_all + !!can + !!map + !!size_only + !!uri_only > 1) {
     fprintf (stderr,
-             "%s: you cannot use --list, --map and --size together.\n",
+             "%s: you cannot use --can, --cannot, --list, --map, --size "
+             "and --uri together.\n",
              progname);
     exit (EXIT_FAILURE);
   }
@@ -277,11 +319,12 @@ main (int argc, char *argv[])
     fprintf (stderr, "%s: %s\n", progname, nbd_get_error ());
     exit (EXIT_FAILURE);
   }
+  atexit (clean_shutdown);
   nbd_set_uri_allow_local_file (nbd, true); /* Allow ?tls-psk-file. */
 
   /* Set optional modes in the handle. */
   nbd_set_opt_mode (nbd, true);
-  if (!can && !map && !size_only)
+  if (!can && !map && !size_only && !uri_only)
     nbd_set_full_info (nbd, true);
   if (map)
     nbd_add_meta_context (nbd, map);
@@ -295,11 +338,13 @@ main (int argc, char *argv[])
 
   if (size_only)                /* --size (!list_all) */
     do_size ();
-  else if (can)                 /* --is/--can/--has (!list_all) */
+  else if (uri_only)            /* --uri (!list_all) */
+    do_uri ();
+  else if (can)                 /* --can/--cannot (!list_all) */
     do_can ();
   else if (map)                 /* --map (!list_all) */
     do_map ();
-  else {                        /* not --size or --map */
+  else {                        /* not --size, --uri, --is or --map */
     const char *protocol;
     int tls_negotiated;
     int sr_negotiated;
@@ -313,7 +358,6 @@ main (int argc, char *argv[])
 
     if (!json_output) {
       if (protocol) {
-        ansi_colour (ANSI_FG_GREY, fp);
         fprintf (fp, "protocol: %s", protocol);
         if (tls_negotiated >= 0)
           fprintf (fp, " %s TLS", tls_negotiated ? "with" : "without");
@@ -322,7 +366,6 @@ main (int argc, char *argv[])
                    eh_negotiated ? "extended" :
                    sr_negotiated ? "structured" : "simple");
         fprintf (fp, "\n");
-        ansi_restore (fp);
       }
     }
     else {
@@ -353,6 +396,7 @@ main (int argc, char *argv[])
   free_exports ();
   nbd_shutdown (nbd, 0);
   nbd_close (nbd);
+  nbd = NULL;
 
   /* Close the output stream and copy it to the real stdout. */
   if (fclose (fp) == EOF) {
