@@ -42,7 +42,7 @@
 
 #define SIZE (1024*1024)
 
-#if CERTS || PSK
+#if defined(CERTS) || defined(PSK)
 #define TLS 1
 #ifndef TLS_MODE
 #error "TLS_MODE must be defined when using CERTS || PSK"
@@ -77,6 +77,7 @@ main (int argc, char *argv[])
   int64_t actual_size;
   char buf[512], buf2[512];
   size_t i;
+  int r;
 
   /* Check requirements or skip the test. */
 #ifdef REQUIRES
@@ -144,15 +145,22 @@ main (int argc, char *argv[])
     fprintf (stderr, "%s\n", nbd_get_error ());
     exit (EXIT_FAILURE);
   }
-#endif
-
-#if CERTS
-  if (nbd_set_tls_certificates (nbd, "../tests/pki") == -1) {
+#ifdef TLS_HOSTNAME
+  if (nbd_set_tls_hostname (nbd, TLS_HOSTNAME) == -1) {
     fprintf (stderr, "%s\n", nbd_get_error ());
     exit (EXIT_FAILURE);
   }
-#elif PSK
-  if (nbd_set_tls_psk_file (nbd, "../tests/keys.psk") == -1) {
+#endif
+#endif /* TLS */
+
+#if defined(CERTS)
+  const char *certs = CERTS;
+  if (certs && nbd_set_tls_certificates (nbd, certs) == -1) {
+    fprintf (stderr, "%s\n", nbd_get_error ());
+    exit (EXIT_FAILURE);
+  }
+#elif defined(PSK)
+  if (nbd_set_tls_psk_file (nbd, PSK) == -1) {
     fprintf (stderr, "%s\n", nbd_get_error ());
     exit (EXIT_FAILURE);
   }
@@ -170,10 +178,21 @@ main (int argc, char *argv[])
 #else
 #define NBD_CONNECT nbd_connect_command
 #endif
-  if (NBD_CONNECT (nbd, args) == -1) {
+  r = NBD_CONNECT (nbd, args);
+#if EXPECT_FAIL
+  if (r != -1) {
+    fprintf (stderr, "%s: expected connection to fail but it did not\n",
+             argv[0]);
+    exit (EXIT_FAILURE);
+  }
+  exit (EXIT_SUCCESS);
+  /*NOTREACHED*/
+#else
+  if (r == -1) {
     fprintf (stderr, "%s\n", nbd_get_error ());
     exit (EXIT_FAILURE);
   }
+#endif
 
 #if TLS
   if (TLS_MODE == LIBNBD_TLS_REQUIRE) {
