@@ -21,6 +21,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <inttypes.h>
 #include <string.h>
 #include <errno.h>
@@ -36,9 +37,11 @@
 #include <sys/vsock.h>
 #endif
 
-#include "internal.h"
+#include "array-size.h"
 #include "vector.h"
 
+#include "internal.h"
+
 #ifdef HAVE_LIBXML2
 
 #include <libxml/uri.h>
@@ -150,12 +153,38 @@ error:
   return -1;
 }
 
+/* Similar to nbdkit_parse_bool */
+int
+parse_bool (const char *param, const char *value)
+{
+  if (!strcmp (value, "1") ||
+      !strcasecmp (value, "true") ||
+      !strcasecmp (value, "t") ||
+      !strcasecmp (value, "yes") ||
+      !strcasecmp (value, "y") ||
+      !strcasecmp (value, "on"))
+    return 1;
+
+  if (!strcmp (value, "0") ||
+      !strcasecmp (value, "false") ||
+      !strcasecmp (value, "f") ||
+      !strcasecmp (value, "no") ||
+      !strcasecmp (value, "n") ||
+      !strcasecmp (value, "off"))
+    return 0;
+
+  set_error (EINVAL, "could not parse %s parameter, expecting %s=true|false",
+             param, param);
+  return -1;
+}
+
 int
 nbd_unlocked_aio_connect_uri (struct nbd_handle *h, const char *raw_uri)
 {
   xmlURIPtr uri = NULL;
-  enum { tcp, unix_sock, vsock } transport;
-  bool tls, socket_required;
+  enum { tcp, unix_sock, vsock, ssh } transport;
+  bool tls;
+  enum { none, optional, required } socket_param;
   uri_query_list queries = empty_vector;
   int i, r;
   int ret = -1;
@@ -176,32 +205,42 @@ nbd_unlocked_aio_connect_uri (struct nbd_handle *h, const char *raw_uri)
     if (strcasecmp (uri->scheme, "nbd") == 0) {
       transport = tcp;
       tls = false;
-      socket_required = false;
+      socket_param = none;
     }
     else if (strcasecmp (uri->scheme, "nbds") == 0) {
       transport = tcp;
       tls = true;
-      socket_required = false;
+      socket_param = none;
     }
     else if (strcasecmp (uri->scheme, "nbd+unix") == 0) {
       transport = unix_sock;
       tls = false;
-      socket_required = true;
+      socket_param = required;
     }
     else if (strcasecmp (uri->scheme, "nbds+unix") == 0) {
       transport = unix_sock;
       tls = true;
-      socket_required = true;
+      socket_param = required;
     }
     else if (strcasecmp (uri->scheme, "nbd+vsock") == 0) {
       transport = vsock;
       tls = false;
-      socket_required = false;
+      socket_param = none;
     }
     else if (strcasecmp (uri->scheme, "nbds+vsock") == 0) {
       transport = vsock;
       tls = true;
-      socket_required = false;
+      socket_param = none;
+    }
+    else if (strcasecmp (uri->scheme, "nbd+ssh") == 0) {
+      transport = ssh;
+      tls = false;
+      socket_param = optional;
+    }
+    else if (strcasecmp (uri->scheme, "nbds+ssh") == 0) {
+      transport = ssh;
+      tls = true;
+      socket_param = optional;
     }
     else {
       set_error (EINVAL, "unknown NBD URI scheme: %s", uri->scheme);
@@ -241,7 +280,9 @@ nbd_unlocked_aio_connect_uri (struct nbd_handle *h, const char *raw_uri)
       (transport == unix_sock &&
        (h->uri_allow_transports & LIBNBD_ALLOW_TRANSPORT_UNIX) == 0) ||
       (transport == vsock &&
-       (h->uri_allow_transports & LIBNBD_ALLOW_TRANSPORT_VSOCK) == 0)) {
+       (h->uri_allow_transports & LIBNBD_ALLOW_TRANSPORT_VSOCK) == 0) ||
+      (transport == ssh &&
+       (h->uri_allow_transports & LIBNBD_ALLOW_TRANSPORT_SSH) == 0)) {
     set_error (EPERM, "URI transport %s is not permitted", uri->scheme);
     goto cleanup;
   }
@@ -259,17 +300,25 @@ nbd_unlocked_aio_connect_uri (struct nbd_handle *h, const char *raw_uri)
       unixsocket = queries.ptr[i].value;
   }
 
-  if (socket_required && !unixsocket) {
-    set_error (EINVAL, "cannot parse socket parameter from NBD URI "
-               "(did you mean to use \"%s:///?socket=...\"?)",
-               uri->scheme);
-    goto cleanup;
-  }
-  else if (!socket_required && unixsocket) {
-    set_error (EINVAL, "socket parameter is incompatible with \"%s:\" "
-               "(did you mean to use \"%s+unix:///?socket=...\"?)",
-               uri->scheme, !tls ? "nbd" : "nbds");
-    goto cleanup;
+  switch (socket_param) {
+  case required:
+    if (!unixsocket) {
+      set_error (EINVAL, "cannot parse socket parameter from NBD URI "
+                 "(did you mean to use \"%s:///?socket=...\"?)",
+                 uri->scheme);
+      goto cleanup;
+    }
+    break;
+  case none:
+    if (unixsocket) {
+      set_error (EINVAL, "socket parameter is incompatible with \"%s:\" "
+                 "(did you mean to use \"%s+unix:///?socket=...\"?)",
+                 uri->scheme, !tls ? "nbd" : "nbds");
+      goto cleanup;
+    }
+    break;
+  case optional:
+    /* no check needed */ ;
   }
 
   /* TLS */
@@ -298,6 +347,17 @@ nbd_unlocked_aio_connect_uri (struct nbd_handle *h, const char *raw_uri)
       if (nbd_unlocked_set_tls_psk_file (h, queries.ptr[i].value) == -1)
         goto cleanup;
     }
+    else if (strcasecmp (queries.ptr[i].name, "tls-hostname") == 0) {
+      if (nbd_unlocked_set_tls_hostname (h, queries.ptr[i].value) == -1)
+        goto cleanup;
+    }
+    else if (strcasecmp (queries.ptr[i].name, "tls-verify-peer") == 0) {
+      int v = parse_bool ("tls-verify-peer", queries.ptr[i].value);
+      if (v == -1)
+        goto cleanup;
+      if (nbd_unlocked_set_tls_verify_peer (h, v) == -1)
+        goto cleanup;
+    }
   }
 
   /* Username. */
@@ -382,6 +442,36 @@ nbd_unlocked_aio_connect_uri (struct nbd_handle *h, const char *raw_uri)
 
     break;
   }
+
+  case ssh: {                   /* SSH */
+    char port_str[32];
+    const char *ssh_command[] = {
+      "ssh", "-p", port_str, uri->server,
+      "nc",
+      NULL,                     /* [5] "-U" or "localhost" */
+      NULL,                     /* [6] socket or "10809" */
+      NULL,
+    };
+
+    if (!uri->server || strcmp (uri->server, "") == 0) {
+      set_error (EINVAL, "SSH transport requires a server name");
+      goto cleanup;
+    }
+    snprintf (port_str, sizeof port_str,
+              "%d", uri->port > 0 ? uri->port : 22);
+
+    if (unixsocket) {
+      ssh_command[5] = "-U";
+      ssh_command[6] = unixsocket;
+    }
+    else {
+      ssh_command[5] = "localhost";
+      ssh_command[6] = "10809"; /* XXX provide a way to configure this */
+    }
+
+    if (nbd_unlocked_aio_connect_command (h, (char **) ssh_command) == -1)
+      goto cleanup;
+  }
   }
 
   ret = 0;
@@ -543,6 +633,16 @@ nbd_unlocked_get_uri (struct nbd_handle *h)
                              "tls-psk-file", h->tls_psk_file) == -1)
       goto out;
   }
+  if (h->tls_hostname) {
+    if (append_query_params (&query_params,
+                             "tls-hostname", h->tls_hostname) == -1)
+      goto out;
+  }
+  if (! h->tls_verify_peer) {
+    if (append_query_params (&query_params,
+                             "tls-verify-peer", "false") == -1)
+      goto out;
+  }
 
   uri.query_raw = query_params;
 
@@ -600,3 +700,28 @@ nbd_unlocked_get_uri (struct nbd_handle *h)
 }
 
 #endif /* !HAVE_LIBXML2 */
+
+/* nbd_is_uri is simple enough that it works even if we don't have URI
+ * support.
+ */
+int
+nbd_unlocked_is_uri (struct nbd_handle *h, const char *uri)
+{
+  static const char *prefixes[] = {
+    "nbd://",
+    "nbds://",
+    "nbd+unix://",
+    "nbds+unix://",
+    "nbd+vsock://",
+    "nbds+vsock://",
+    "nbd+ssh://",
+    "nbds+ssh://",
+  };
+  size_t i;
+
+  for (i = 0; i < ARRAY_SIZE (prefixes); ++i) {
+    if (strncasecmp (uri, prefixes[i], strlen (prefixes[i])) == 0)
+      return 1;
+  }
+  return 0;
+}
