@@ -20,12 +20,14 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
+#include <stdarg.h>
 #include <string.h>
+#include <fcntl.h>
 #include <unistd.h>
 #include <ctype.h>
 #include <errno.h>
-#include <fcntl.h>
-#include <stdarg.h>
+#include <assert.h>
 #include <sys/uio.h>
 
 #include "array-size.h"
