@@ -21,10 +21,13 @@
 set -e
 set -x
 
-requires nbdkit --version
+requires $NBDKIT --version
 requires $PYTHON --version
 
-nbdkit -U - null --run '$PYTHON -c "
+# Python tests all fail on macOS because of SIP misfeature.
+requires_not test "$(uname)" = "Darwin"
+
+$NBDKIT -U - null --run '$PYTHON -c "
 import nbd
 import sys
 h = nbd.NBD()
