@@ -18,7 +18,11 @@
 
 # Test AIO parallel data integrity.
 
-nbdkit -U - \
+source ./functions.sh
+set -e
+set -x
+
+$NBDKIT -U - \
        --filter=cow \
        pattern size=64M \
        --run '$VG ./aio-parallel $unixsocket'
