@@ -21,10 +21,13 @@
 
 extern void requires (const char *cmd);
 extern void requires_not (const char *cmd);
+extern void requires_not_exists (const char *filename);
 
 /* Some specific tests using the requires() mechanism. */
 extern void requires_qemu_nbd_tls_support (void);
 extern void requires_qemu_nbd_tls_psk_support (void);
 extern void requires_nbd_server_supports_inetd (void);
+extern void requires_nbdkit_tls_verify_peer (void);
+extern void requires_ssh_localhost (void);
 
 #endif /* LIBNBD_REQUIRES */
