@@ -18,7 +18,11 @@
 
 # Test synchronous parallel high level API requests.
 
-nbdkit -U - \
-       --filter=cow \
-       pattern size=8M \
-       --run '$VG ./synch-parallel $unixsocket'
+source ./functions.sh
+set -e
+set -x
+
+$NBDKIT -U - \
+        --filter=cow \
+        pattern size=8M \
+        --run '$VG ./synch-parallel $unixsocket'
